import arcpy, os.path, sys

versionString = 'NCMGP09v1.1_SetSymbols_Arc10.1.py, version of 9 September 2013'

editPrefixes = ('xxx','edit_','errors_')

EightfoldLineDict = {}
TwofoldOrientPointDict = {}
MySymbolDict = {}

unrecognizedTypes = []

dictionaryFile = os.path.dirname(sys.argv[0])+'/Type-FgdcSymbol.txt'

def getCaf(inFds):
    arcpy.env.workspace = inFds
    fcs = arcpy.ListFeatureClasses()
    cafs = []
    for fc in fcs:
        if fc.find('ContactsAndFaults') > -1 or (inFds.find('CorrelationOfMapUnits') > -1 and fc.find('Lines') > -1):
            cafs.append(fc)
    for fc in cafs:
        for pfx in editPrefixes:
            if fc.find(pfx) > -1:  # no prefix
                cafs.remove(fc)
    cafs2 = []
    for fc in cafs:
        if fc[-17:] == 'ContactsAndFaults' or (inFds.find('CorrelationOfMapUnits') > -1 and fc[-5:] == 'Lines'):
            cafs2.append(fc)
    #addMsgAndPrint(str(cafs))
    if len(cafs2) <> 1:
        addMsgAndPrint('  Cannot resolve ContactsAndFaults feature class in feature dataset')
        addMsgAndPrint('    '+inFds)
        addMsgAndPrint('    '+str(cafs2))
        raise arcpy.ExecuteError
    return inFds+'/'+cafs2[0]

def addMsgAndPrint(msg, severity=0): 
    # prints msg to screen and adds msg to the geoprocessor (in case this is run as a tool) 
    try: 
      for string in msg.split('\n'): 
        # Add appropriate geoprocessing message 
        if severity == 0: 
            arcpy.AddMessage(string) 
        elif severity == 1: 
            arcpy.AddWarning(string) 
        elif severity == 2: 
            arcpy.AddError(string) 
    except: 
        pass

def buildSymbolDicts(dFile):
    df = open(dictionaryFile,'r')
    for aline in df:
        if aline[0] <> '#':
            words = aline.split()
            if len(words) > 0:
                aline = aline[:-1]
                newWords = []
                words = aline.split('|')
                for word in words:
                    newWord = word.lstrip().rstrip()
                    newWords.append(newWord)
                if aline == '***Eight-fold Lines***':
                    aDict = EightfoldLineDict
                elif aline == '***Two-fold Orientation Points***':
                    aDict = TwofoldOrientPointDict
                elif aline == '***My Symbols***':
                    aDict = MySymbolDict
                else:
                    key = newWords[0]
                    val1 = newWords[1]
                    if len(newWords) > 2:
                        val2 = newWords[2]
                        aDict[key] = [val1,val2]
                    else:
                        aDict[key] = val1
    
def isQuestionable(conf):
    if conf.lower() <> 'certain' and conf.lower() <> 'unspecified':
        return True
    else:
        return False

def incrementSymbol(sym,increment):
    symWords = sym.split('.')
    lastWord = symWords[len(symWords)-1]
    newLastWord = str(int(lastWord)+increment).zfill(len(lastWord))
    newSym = sym[0:0-len(lastWord)]+newLastWord
    return newSym

def unrecognizedType(t):
    if not t in unrecognizedTypes:
        unrecognizedTypes.append(t)

def hasCartoRep(fds,fc):
    hostGdb = os.path.dirname(fds)
    domains = arcpy.da.ListDomains(hostGdb)
    shortFc = os.path.basename(fc)
    hasRep = False
    repDomain = ''
    for aDomain in domains:
        if aDomain.name.find(shortFc)>-1 and aDomain.name.find('_rep')>-1:
            hasRep = True
            repDomain = aDomain
    return hasRep, repDomain

def trimLeftZeros(fgdc):
    words = fgdc.split('.')
    fgdc1 = ''
    for word in words:
        fgdc1 = fgdc1+'.'+str(int(word))
    fgdc2 = fgdc1[1:]
    return fgdc2

def buildRepRuleDict(repDomain):
    newDict = {}
    domKeys = list(repDomain.codedValues.keys())
    domKeys.sort()
    for i in domKeys:
        newDict[repDomain.codedValues[i]] = i
    return newDict

#####################################################

# get inputs
inFds = sys.argv[1]
mapScale = float(sys.argv[2])
certain_Approxmm = float(sys.argv[3])
if sys.argv[4] == 'true':
    useInferred = True
else:
    useInferred = False
if useInferred:
    approx_Inferredmm = float(sys.argv[5])
else:
    approx_Inferredmm = 100
if sys.argv[6] == 'true':
    useApproxOrient = True
else:
    useApproxOrient = False
orientThresholdDegrees = float(sys.argv[7])

# set thresholds
approxThreshold = mapScale * certain_Approxmm / 1000.0
inferredThreshold = mapScale * approx_Inferredmm / 1000.0
#read dictionaryFile to build symbolDicts
buildSymbolDicts(dictionaryFile)
#set featureClasses  (ContactsAndFaults, OrientationPoints, GeologicLines)
caf = getCaf(inFds)
gel = caf.replace('ContactsAndFaults','GeologicLines')
orp = caf.replace('ContactsAndFaults','OrientationPoints')
fields = ['Type','IsConcealed','LocationConfidenceMeters','ExistenceConfidence','IdentityConfidence','Symbol']
for fc in caf,gel:
    if arcpy.Exists(fc):
        addMsgAndPrint('  processing '+os.path.basename(fc))
        hasRep, repDomain = hasCartoRep(inFds,fc)
        if hasRep:
            fields.append('RuleID1')
            repRuleDict = buildRepRuleDict(repDomain)
        with arcpy.da.UpdateCursor(fc, fields) as cursor:
            for row in cursor:
                rowChanged = False
                typ = row[0]
                isCon = row[1]
                locConfM = row[2]
                exConf = row[3]
                idConf = row[4]
                if typ in EightfoldLineDict.keys():
                    inc = 0                     
                    if isQuestionable(exConf) or isQuestionable(idConf):
                        inc = inc+1
                    if isCon == 'N':
                        if useInferred and locConfM > inferredThreshold:
                            inc = inc+4
                        elif locConfM > approxThreshold:
                            inc = inc+2
                    else: # isCon == 'Y'
                        inc = inc+6
                    row[5] = incrementSymbol(EightfoldLineDict[typ],inc)
                    rowChanged = True
                elif typ in MySymbolDict.keys():
                    row[5] = MySymbolDict[typ]
                    rowChanged = True
                else:
                    unrecognizedType(typ)
                if rowChanged:
                    if hasRep:
                        # turn GSC label into original FGDC labe: 06.03 to 6.3
                        noZeros = trimLeftZeros(row[5])
                        if repRuleDict.has_key(noZeros):
                            row[6] = repRuleDict[noZeros]
                    cursor.updateRow(row)     
                    
fields = ['Type','OrientationConfidenceDegrees','Symbol']
fc = orp
if arcpy.Exists(fc):
    addMsgAndPrint('  processing '+os.path.basename(fc))
    hasRep, repDomain = hasCartoRep(inFds,fc)
    if hasRep:
        fields.append('RuleID1')
        repRuleDict = buildRepRuleDict(repDomain)
    with arcpy.da.UpdateCursor(fc, fields) as cursor:
        for row in cursor:
            typ = row[0]
            orConf = row[1]
            rowChanged = False
            if typ in TwofoldOrientPointDict.keys():
                rowChanged = True
                if orConf > orientThresholdDegrees and useApproxOrient:
                    row[2] = TwofoldOrientPointDict[typ][1]
                else:
                    row[2] = TwofoldOrientPointDict[typ][0]
            else:
                unrecognizedType(typ)
            if rowChanged:
                if hasRep:
                    # turn GSC label into original FGDC label: 06.03 to 6.3
                    noZeros = trimLeftZeros(row[2])
                    if repRuleDict.has_key(noZeros):
                        row[3] = repRuleDict[noZeros]
                cursor.updateRow(row)

addMsgAndPrint('  \n  Unrecognized Type values: ')
if len(unrecognizedTypes) == 0:
    addMsgAndPrint('    none')
else:
    for t in unrecognizedTypes:
        addMsgAndPrint('    '+t)
addMsgAndPrint('  ')

#raise arcpy.ExecuteError

